"""Generates submodule to allow deprecation of submodules and keeping git
blame."""
from pathlib import Path
from contextlib import suppress

# TODO: Remove the whole file in 0.24

# This is a set of 4-tuples consisting of
# (new_module_name, deprecated_path, correct_import_path, importee)
# importee is used by test_import_deprecations to check for DeprecationWarnings
_DEPRECATED_MODULES = [
    ('_mocking', 'pai4sk.utils.mocking', 'pai4sk.utils',
     'MockDataFrame'),

    ('_bagging', 'pai4sk.ensemble.bagging', 'pai4sk.ensemble',
     'BaggingClassifier'),
    ('_base', 'pai4sk.ensemble.base', 'pai4sk.ensemble',
     'BaseEnsemble'),
    ('_forest', 'pai4sk.ensemble.forest', 'pai4sk.ensemble',
     'RandomForestClassifierSklearn'),
    ('_gb', 'pai4sk.ensemble.gradient_boosting', 'pai4sk.ensemble',
     'GradientBoostingClassifier'),
    ('_iforest', 'pai4sk.ensemble.iforest', 'pai4sk.ensemble',
     'IsolationForest'),
    ('_voting', 'pai4sk.ensemble.voting', 'pai4sk.ensemble',
     'VotingClassifier'),
    ('_weight_boosting', 'pai4sk.ensemble.weight_boosting',
     'pai4sk.ensemble', 'AdaBoostClassifier'),
    ('_classes', 'pai4sk.tree.tree', 'pai4sk.tree',
     'DecisionTreeClassifier'),
    ('_export', 'pai4sk.tree.export', 'pai4sk.tree', 'export_graphviz'),

    ('_rbm', 'pai4sk.neural_network.rbm', 'pai4sk.neural_network',
     'BernoulliRBM'),
    ('_multilayer_perceptron', 'pai4sk.neural_network.multilayer_perceptron',
     'pai4sk.neural_network', 'MLPClassifier'),

    ('_weight_vector', 'pai4sk.utils.weight_vector', 'pai4sk.utils',
     'WeightVector'),
    ('_seq_dataset', 'pai4sk.utils.seq_dataset', 'pai4sk.utils',
     'ArrayDataset32'),
    ('_fast_dict', 'pai4sk.utils.fast_dict', 'pai4sk.utils', 'IntFloatDict'),

    ('_affinity_propagation', 'pai4sk.cluster.affinity_propagation_',
     'pai4sk.cluster', 'AffinityPropagation'),
    ('_bicluster', 'pai4sk.cluster.bicluster', 'pai4sk.cluster',
     'SpectralBiclustering'),
    ('_birch', 'pai4sk.cluster.birch', 'pai4sk.cluster', 'Birch'),
    ('_dbscan', 'pai4sk.cluster.dbscan_', 'pai4sk.cluster', 'DBSCANSklearn'),
    ('_agglomerative', 'pai4sk.cluster.hierarchical', 'pai4sk.cluster',
     'FeatureAgglomeration'),
    ('_kmeans', 'pai4sk.cluster.k_means_', 'pai4sk.cluster', 'KMeansSklearn'),
    ('_mean_shift', 'pai4sk.cluster.mean_shift_', 'pai4sk.cluster',
     'MeanShift'),
    ('_optics', 'pai4sk.cluster.optics_', 'pai4sk.cluster', 'OPTICS'),
    ('_spectral', 'pai4sk.cluster.spectral', 'pai4sk.cluster',
     'SpectralClustering'),

    ('_base', 'pai4sk.mixture.base', 'pai4sk.mixture', 'BaseMixture'),
    ('_gaussian_mixture', 'pai4sk.mixture.gaussian_mixture',
     'pai4sk.mixture', 'GaussianMixture'),
    ('_bayesian_mixture', 'pai4sk.mixture.bayesian_mixture',
     'pai4sk.mixture', 'BayesianGaussianMixture'),

    ('_empirical_covariance', 'pai4sk.covariance.empirical_covariance_',
     'pai4sk.covariance', 'EmpiricalCovariance'),
    ('_shrunk_covariance', 'pai4sk.covariance.shrunk_covariance_',
     'pai4sk.covariance', 'ShrunkCovariance'),
    ('_robust_covariance', 'pai4sk.covariance.robust_covariance',
     'pai4sk.covariance', 'MinCovDet'),
    ('_graph_lasso', 'pai4sk.covariance.graph_lasso_',
     'pai4sk.covariance', 'GraphicalLasso'),
    ('_elliptic_envelope', 'pai4sk.covariance.elliptic_envelope',
     'pai4sk.covariance', 'EllipticEnvelope'),

    ('_cca', 'pai4sk.cross_decomposition.cca_',
     'pai4sk.cross_decomposition', 'CCA'),
    ('_pls', 'pai4sk.cross_decomposition.pls_',
     'pai4sk.cross_decomposition', 'PLSSVD'),

    ('_base', 'pai4sk.svm.base', 'pai4sk.svm', 'BaseLibSVM'),
    ('_bounds', 'pai4sk.svm.bounds', 'pai4sk.svm', 'l1_min_c'),
    ('_classes', 'pai4sk.svm.classes', 'pai4sk.svm', 'SVR'),
    ('_libsvm', 'pai4sk.svm.libsvm', 'pai4sk.svm', 'fit'),
    ('_libsvm_sparse', 'pai4sk.svm.libsvm_sparse', 'pai4sk.svm',
     'set_verbosity_wrap'),
    ('_liblinear', 'pai4sk.svm.liblinear', 'pai4sk.svm', 'train_wrap'),

    ('_base', 'pai4sk.decomposition.base', 'pai4sk.decomposition',
     'BaseEstimator'),
    ('_dict_learning', 'pai4sk.decomposition.dict_learning',
     'pai4sk.decomposition', 'MiniBatchDictionaryLearning'),
    ('_cdnmf_fast', 'pai4sk.decomposition.cdnmf_fast',
     'pai4sk.decomposition', '__dict__'),
    ('_factor_analysis', 'pai4sk.decomposition.factor_analysis',
     'pai4sk.decomposition', 'FactorAnalysis'),
    ('_fastica', 'pai4sk.decomposition.fastica_', 'pai4sk.decomposition',
     'FastICA'),
    ('_incremental_pca', 'pai4sk.decomposition.incremental_pca',
     'pai4sk.decomposition', 'IncrementalPCA'),
    ('_kernel_pca', 'pai4sk.decomposition.kernel_pca',
     'pai4sk.decomposition', 'KernelPCA'),
    ('_nmf', 'pai4sk.decomposition.nmf', 'pai4sk.decomposition', 'NMF'),
    ('_lda', 'pai4sk.decomposition.online_lda',
     'pai4sk.decomposition', 'LatentDirichletAllocation'),
    ('_online_lda_fast', 'pai4sk.decomposition.online_lda_fast',
     'pai4sk.decomposition', 'mean_change'),
    ('_pca', 'pai4sk.decomposition.pca', 'pai4sk.decomposition', 'PCASklearn'),
    ('_sparse_pca', 'pai4sk.decomposition.sparse_pca',
     'pai4sk.decomposition', 'SparsePCA'),
    ('_truncated_svd', 'pai4sk.decomposition.truncated_svd',
     'pai4sk.decomposition', 'TruncatedSVDSklearn'),

    ('_gpr', 'pai4sk.gaussian_process.gpr', 'pai4sk.gaussian_process',
     'GaussianProcessRegressor'),
    ('_gpc', 'pai4sk.gaussian_process.gpc', 'pai4sk.gaussian_process',
     'GaussianProcessClassifier'),

    ('_base', 'pai4sk.datasets.base', 'pai4sk.datasets', 'get_data_home'),
    ('_california_housing', 'pai4sk.datasets.california_housing',
     'pai4sk.datasets', 'fetch_california_housing'),
    ('_covtype', 'pai4sk.datasets.covtype', 'pai4sk.datasets',
     'fetch_covtype'),
    ('_kddcup99', 'pai4sk.datasets.kddcup99', 'pai4sk.datasets',
     'fetch_kddcup99'),
    ('_lfw', 'pai4sk.datasets.lfw', 'pai4sk.datasets',
     'fetch_lfw_people'),
    ('_olivetti_faces', 'pai4sk.datasets.olivetti_faces', 'pai4sk.datasets',
     'fetch_olivetti_faces'),
    ('_openml', 'pai4sk.datasets.openml', 'pai4sk.datasets', 'fetch_openml'),
    ('_rcv1', 'pai4sk.datasets.rcv1', 'pai4sk.datasets', 'fetch_rcv1'),
    ('_samples_generator', 'pai4sk.datasets.samples_generator',
     'pai4sk.datasets', 'make_classification'),
    ('_species_distributions', 'pai4sk.datasets.species_distributions',
     'pai4sk.datasets', 'fetch_species_distributions'),
    ('_svmlight_format_io', 'pai4sk.datasets.svmlight_format',
     'pai4sk.datasets', 'load_svmlight_fileSklearn'),
    ('_twenty_newsgroups', 'pai4sk.datasets.twenty_newsgroups',
     'pai4sk.datasets', 'strip_newsgroup_header'),

    ('_dict_vectorizer', 'pai4sk.feature_extraction.dict_vectorizer',
     'pai4sk.feature_extraction', 'DictVectorizer'),
    ('_hash', 'pai4sk.feature_extraction.hashing',
     'pai4sk.feature_extraction', 'FeatureHasher'),
    ('_stop_words', 'pai4sk.feature_extraction.stop_words',
     'pai4sk.feature_extraction.text', 'ENGLISH_STOP_WORDS'),

    ('_base', 'pai4sk.linear_model.base', 'pai4sk.linear_model',
     'LinearRegression'),
    ('_cd_fast', 'pai4sk.linear_model.cd_fast', 'pai4sk.linear_model',
     'sparse_enet_coordinate_descent'),
    ('_bayes', 'pai4sk.linear_model.bayes', 'pai4sk.linear_model',
     'BayesianRidge'),
    ('_coordinate_descent', 'pai4sk.linear_model.coordinate_descent',
     'pai4sk.linear_model', 'LassoSklearn'),
    ('_huber', 'pai4sk.linear_model.huber', 'pai4sk.linear_model',
     'HuberRegressor'),
    ('_least_angle', 'pai4sk.linear_model.least_angle',
     'pai4sk.linear_model', 'LassoLarsCV'),
    ('_logistic', 'pai4sk.linear_model.logistic', 'pai4sk.linear_model',
     'LogisticRegressionSklearn'),
    ('_omp', 'pai4sk.linear_model.omp', 'pai4sk.linear_model',
     'OrthogonalMatchingPursuit'),
    ('_passive_aggressive', 'pai4sk.linear_model.passive_aggressive',
     'pai4sk.linear_model', 'PassiveAggressiveClassifier'),
    ('_perceptron', 'pai4sk.linear_model.perceptron', 'pai4sk.linear_model',
     'Perceptron'),
    ('_ransac', 'pai4sk.linear_model.ransac', 'pai4sk.linear_model',
     'RANSACRegressor'),
    ('_ridge', 'pai4sk.linear_model.ridge', 'pai4sk.linear_model',
     'RidgeSklearn'),
    ('_sag', 'pai4sk.linear_model.sag', 'pai4sk.linear_model',
     'get_auto_step_size'),
    ('_sag_fast', 'pai4sk.linear_model.sag_fast', 'pai4sk.linear_model',
     'MultinomialLogLoss64'),
    ('_sgd_fast', 'pai4sk.linear_model.sgd_fast', 'pai4sk.linear_model',
     'Hinge'),
    ('_stochastic_gradient', 'pai4sk.linear_model.stochastic_gradient',
     'pai4sk.linear_model', 'SGDClassifier'),
    ('_theil_sen', 'pai4sk.linear_model.theil_sen', 'pai4sk.linear_model',
     'TheilSenRegressor'),

    ('_bicluster', 'pai4sk.metrics.cluster.bicluster',
     'pai4sk.metrics.cluster', 'consensus_score'),
    ('_supervised', 'pai4sk.metrics.cluster.supervised',
     'pai4sk.metrics.cluster', 'entropy'),
    ('_unsupervised', 'pai4sk.metrics.cluster.unsupervised',
     'pai4sk.metrics.cluster', 'silhouette_score'),
    ('_expected_mutual_info_fast',
     'pai4sk.metrics.cluster.expected_mutual_info_fast',
     'pai4sk.metrics.cluster', 'expected_mutual_information'),

    ('_base', 'pai4sk.metrics.base', 'pai4sk.metrics', 'combinations'),
    ('_classification', 'pai4sk.metrics.classification', 'pai4sk.metrics',
     'accuracy_scoreSklearn'),
    ('_regression', 'pai4sk.metrics.regression', 'pai4sk.metrics',
     'max_error'),
    ('_ranking', 'pai4sk.metrics.ranking', 'pai4sk.metrics', 'roc_curve'),
    ('_pairwise_fast', 'pai4sk.metrics.pairwise_fast', 'pai4sk.metrics',
     'np'),
    ('_scorer', 'pai4sk.metrics.scorer', 'pai4sk.metrics', 'get_scorer'),

    ('_partial_dependence', 'pai4sk.inspection.partial_dependence',
     'pai4sk.inspection', 'partial_dependence'),

    ('_ball_tree', 'pai4sk.neighbors.ball_tree', 'pai4sk.neighbors',
     'BallTree'),
    ('_base', 'pai4sk.neighbors.base', 'pai4sk.neighbors',
     'VALID_METRICS'),
    ('_classification', 'pai4sk.neighbors.classification',
     'pai4sk.neighbors', 'KNeighborsClassifier'),
    ('_dist_metrics', 'pai4sk.neighbors.dist_metrics', 'pai4sk.neighbors',
     'DistanceMetric'),
    ('_graph', 'pai4sk.neighbors.graph', 'pai4sk.neighbors',
     'KNeighborsTransformer'),
    ('_kd_tree', 'pai4sk.neighbors.kd_tree', 'pai4sk.neighbors',
     'KDTree'),
    ('_kde', 'pai4sk.neighbors.kde', 'pai4sk.neighbors',
     'KernelDensity'),
    ('_lof', 'pai4sk.neighbors.lof', 'pai4sk.neighbors',
     'LocalOutlierFactor'),
    ('_nca', 'pai4sk.neighbors.nca', 'pai4sk.neighbors',
     'NeighborhoodComponentsAnalysis'),
    ('_nearest_centroid', 'pai4sk.neighbors.nearest_centroid',
     'pai4sk.neighbors', 'NearestCentroid'),
    ('_quad_tree', 'pai4sk.neighbors.quad_tree', 'pai4sk.neighbors',
     'CELL_DTYPE'),
    ('_regression', 'pai4sk.neighbors.regression', 'pai4sk.neighbors',
     'KNeighborsRegressor'),
    ('_typedefs', 'pai4sk.neighbors.typedefs', 'pai4sk.neighbors',
     'DTYPE'),
    ('_unsupervised', 'pai4sk.neighbors.unsupervised', 'pai4sk.neighbors',
     'NearestNeighborsSklearn'),

    ('_isomap', 'pai4sk.manifold.isomap', 'pai4sk.manifold', 'Isomap'),
    ('_locally_linear', 'pai4sk.manifold.locally_linear', 'pai4sk.manifold',
     'LocallyLinearEmbedding'),
    ('_mds', 'pai4sk.manifold.mds', 'pai4sk.manifold', 'MDS'),
    ('_spectral_embedding', 'pai4sk.manifold.spectral_embedding_',
     'pai4sk.manifold', 'SpectralEmbedding'),
    ('_t_sne', 'pai4sk.manifold.t_sne', 'pai4sk.manifold', 'TSNE'),

    ('_label_propagation', 'pai4sk.semi_supervised.label_propagation',
     'pai4sk.semi_supervised', 'LabelPropagation'),

    ('_data', 'pai4sk.preprocessing.data', 'pai4sk.preprocessing',
     'Binarizer'),
    ('_label', 'pai4sk.preprocessing.label', 'pai4sk.preprocessing',
     'LabelEncoder'),

    ('_base', 'pai4sk.feature_selection.base', 'pai4sk.feature_selection',
     'SelectorMixin'),
    ('_from_model', 'pai4sk.feature_selection.from_model',
     'pai4sk.feature_selection', 'SelectFromModel'),
    ('_mutual_info', 'pai4sk.feature_selection.mutual_info',
     'pai4sk.feature_selection', 'mutual_info_regression'),
    ('_rfe', 'pai4sk.feature_selection.rfe',
     'pai4sk.feature_selection.rfe', 'RFE'),
    ('_univariate_selection',
     'pai4sk.feature_selection.univariate_selection',
     'pai4sk.feature_selection', 'chi2'),
    ('_variance_threshold',
     'pai4sk.feature_selection.variance_threshold',
     'pai4sk.feature_selection', 'VarianceThreshold'),

    ('_testing', 'pai4sk.utils.testing', 'pai4sk.utils',
     'all_estimators'),
]


_FILE_CONTENT_TEMPLATE = """
# THIS FILE WAS AUTOMATICALLY GENERATED BY deprecated_modules.py
import sys
from . import {new_module_name}
from {relative_dots}externals._pep562 import Pep562
from {relative_dots}utils.deprecation import _raise_dep_warning_if_not_pytest

deprecated_path = '{deprecated_path}'
correct_import_path = '{correct_import_path}'

_raise_dep_warning_if_not_pytest(deprecated_path, correct_import_path)

def __getattr__(name):
    return getattr({new_module_name}, name)

if not sys.version_info >= (3, 7):
    Pep562(__name__)
"""


def _get_deprecated_path(deprecated_path):
    deprecated_parts = deprecated_path.split(".")
    deprecated_parts[-1] = deprecated_parts[-1] + ".py"
    return Path(*deprecated_parts)


def _create_deprecated_modules_files():
    """Add submodules that will be deprecated. A file is created based
    on the deprecated submodule's name. When this submodule is imported a
    deprecation warning will be raised.
    """
    for (new_module_name, deprecated_path,
         correct_import_path, _) in _DEPRECATED_MODULES:
        relative_dots = deprecated_path.count(".") * "."
        deprecated_content = _FILE_CONTENT_TEMPLATE.format(
            new_module_name=new_module_name,
            relative_dots=relative_dots,
            deprecated_path=deprecated_path,
            correct_import_path=correct_import_path)

        with _get_deprecated_path(deprecated_path).open('w') as f:
            f.write(deprecated_content)


def _clean_deprecated_modules_files():
    """Removes submodules created by _create_deprecated_modules_files."""
    for _, deprecated_path, _, _ in _DEPRECATED_MODULES:
        with suppress(FileNotFoundError):
            _get_deprecated_path(deprecated_path).unlink()


if __name__ == "__main__":
    _clean_deprecated_modules_files()
