"""
The :mod:`pai4sk.decomposition` module includes matrix decomposition
algorithms, including among others PCASklearn, NMF or ICA. Most of the algorithms of
this module can be regarded as dimensionality reduction techniques.
"""

# TODO: remove me in 0.24 (as well as the noqa markers) and
# import the dict_learning func directly from the ._dict_learning
# module instead.
# Pre-cache the import of the deprecated module so that import
# pai4sk.decomposition.dict_learning returns the function as in
# 0.21, instead of the module.
# https://github.com/pai4sk/pai4sk/issues/15842
import warnings
with warnings.catch_warnings():
    warnings.simplefilter("ignore", category=FutureWarning)
    from .dict_learning import dict_learning


from ._nmf import NMF, non_negative_factorization  # noqa
from ._pca import PCASklearn  # noqa
from ._incremental_pca import IncrementalPCA  # noqa
from ._kernel_pca import KernelPCA  # noqa
from ._sparse_pca import SparsePCA, MiniBatchSparsePCA  # noqa
from ._truncated_svd import TruncatedSVDSklearn  # noqa
from ._fastica import FastICA, fastica  # noqa
from ._dict_learning import (dict_learning_online,
                             sparse_encode, DictionaryLearning,
                             MiniBatchDictionaryLearning, SparseCoder)  # noqa
from ._factor_analysis import FactorAnalysis  # noqa
from ..utils.extmath import randomized_svd  # noqa
from ._lda import LatentDirichletAllocation  # noqa


__all__ = ['DictionaryLearning',
           'FastICA',
           'IncrementalPCA',
           'KernelPCA',
           'MiniBatchDictionaryLearning',
           'MiniBatchSparsePCA',
           'NMF',
           'PCASklearn',
           'SparseCoder',
           'SparsePCA',
           'dict_learning',
           'dict_learning_online',
           'fastica',
           'non_negative_factorization',
           'randomized_svd',
           'sparse_encode',
           'FactorAnalysis',
           'TruncatedSVDSklearn',
           'LatentDirichletAllocation']
import inspect
if '_cuml/solvers/_pca' not  in  [item[1]  for item in inspect.stack() if '<frozen importlib._bootstrap' not  in str(item[1]) ] [1] : from .._cuml.solvers._pca import PCA
if '_cuml/solvers/_truncated_svd' not  in  [item[1]  for item in inspect.stack() if '<frozen importlib._bootstrap' not  in str(item[1]) ] [1] : from .._cuml.solvers._truncated_svd import TruncatedSVD
