# Epsilon Example using CuPy array with SnapML and compare with RAPIDS cuML

SnapML currently supports `numba` `DeviceNDArray` as input for the SnapML linear models.

[CuPy](https://docs-cupy.chainer.org/en/stable/index.html) is a numpy compatible array on the GPU. Using numba API `numba.cuda.as_cuda_array()` we are able to convert `CuPy` array to `DeviceNDArray` without any extra copy of data.

In this example we will train Support Vector Machine model on the epsilon dataset. We will convert the `CuPy` array to `DeviceNDArray` and compare the training time of `SnapML` with RAPIDS `cuML`.

The epsilon dataset is from the [PASCAL Large Scale Learning Challenge](http://www.k4all.org/project/large-scale-learning-challenge/). 


## Download the Data

Download and decompress the data from the LIBSVM repository:
```
mkdir data
cd data
wget https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/binary/epsilon_normalized.bz2
bunzip2 epsilon_normalized.bz2
cd ../
```

## Data Pre-processing
The data is in SvmLight format which is not really appropriate since this dataset is dense. Therefore, we perform some pre-processing. 

The entire pre-processing can be run by executing `$CONDA_PREFIX/pai4sk/local-examples/cupy-examples/preprocess-epsilon.py`. Command to pre-process the data available in specific directory:

`python $CONDA_PREFIX/pai4sk/local-examples/cupy-examples/preprocess-epsilon.py  --data_path path/to/data`

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

## Training and Evaluating a Support Vector Machine Model

One can run the entire example by executing `$CONDA_PREFIX/pai4sk/local-examples/cupy-examples/example-epsilon-cupy-svm.py`. Command to run example using the pre-processed data in specific directory:

`python $CONDA_PREFIX/pai4sk/local-examples/cupy-examples/example-epsilon-cupy-svm.py --data_path path/to/data`

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

Command to run example on GPU 0:
`python $CONDA_PREFIX/pai4sk/local-examples/cupy-examples/example-epsilon-cupy-svm.py`

One should see the the following output on a P9+V100 system:

```
Data load time (s):  1.71
[Info] If set num_threads should be a multiple of 32. GPU training will run with num_threads=256.
[Warning] Parameter device_ids not set. The training will run on GPU 0.
[Info] Tolerance 0.001 attained after 112 epochs.
[SnapML] Training time (s): 1.15
[SnapML] Accuracy : 0.898
[SnapML] Hinge-loss : 0.273
[cuML] Training time (s):  44.29
[cuML] Accuracy: 0.898
[cuML] Hinge loss: 0.273
```

Times will vary depending on system architecture and load.

&copy; Copyright IBM Corporation 2020
