# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************
from pai4sk.datasets import load_svmlight_file
from pai4sk.model_selection import train_test_split
import scipy as sp
import argparse
from pandas import read_csv
import scipy as sp
import numpy as np
import pandas as pd
import time 

defaultPath = "."
CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)
args = CLI.parse_args()

# read the input datasets
t0 = time.time()
data = read_csv(args.data_path + "/data/train.txt", delimiter="\t", header=None)
print("Time to read the input dataset : ", time.time()-t0)
print("Input dataset shape : ", data.shape)

# split input dataset into train and test datasets
data_train, data_test = train_test_split(data, test_size=0.25, random_state=42)
data = None

print("Handling the missing values in the train dataset")
# the first 13 columns are integer features.
# we handle the missing values.
for col in range(1, 14):
    avg_col = data_train[col].mean()
    data_train[col] = data_train[col].fillna(int(avg_col))
    data_test[col]  = data_test[col].fillna(int(avg_col))

print("Handling the missing values in the test dataset")
# the last 26 columns are categorical features.
# we handle the missing values.
for col in range(14,40):
    data_train[col] = data_train[col].fillna("NoCateg")
    data_test[col] = data_test[col].fillna("NoCateg")
    
# replace all categories in the test dataset that 
# do not exist in the train dataset with avg_train_label
avg_train_label = data_train[0].mean()

print("Performing target encoding for both train and test datasets")    
# the last 26 columns are categorical features.
# to convert them to numerical features, we use target encoding (as an example)
for col in range(14,40):
    means = data_train.groupby(col)[0].mean()
    data_train[col] = data_train[col].map(means)
    data_test[col] = data_test[col].map(means)
    data_test[col] = data_test[col].fillna(avg_train_label)

print("Generating the numpy arrays for X_train, X_test, y_train and y_test")
# the last 39 columns represent the features
X_train = data_train.iloc[:,1:40].values
print(X_train.shape)
X_test  = data_test.iloc[:,1:40].values
print(X_test.shape)

# the first column represents the labels
y_train = data_train.iloc[:,0].values
y_test  = data_test.iloc[:,0].values
print(y_train.shape)
print(y_test.shape)

print("Saving the train and test datasets to disk.")
# Save the feature matrices
np.save(args.data_path + "/data/criteo.kaggle.raw.encoded.X_train", X_train)
np.save(args.data_path + "/data/criteo.kaggle.raw.encoded.X_test",  X_test)

# Save the labels
np.save(args.data_path + "/data/criteo.kaggle.raw.encoded.y_train", y_train)
np.save(args.data_path + "/data/criteo.kaggle.raw.encoded.y_test", y_test)

