# HIGGS Example

In this example we will train Logistic Regression, Decision Tree and Random Forest models on the HIGGS dataset, using both `scikit-learn` and `snap-ml-local`. 

The HIGGS dataset is avaliable in the [UCI machine learning repository](https://archive.ics.uci.edu/ml/datasets/HIGGS). 


## Download the Data

Download and decompress the data from the LIBSVM repository:
```
mkdir data
cd data
wget https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/binary/HIGGS.bz2
bunzip2 HIGGS.bz2
cd ../
```

## PreProcess the data

The data is in SvmLight format which is not really appropriate since this dataset is dense. Therefore, we perform 
some pre-processing.

The entire pre-processing can be run by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/preprocess-higgs.py`

Command to pre-process the data available in specific directory:
`python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/preprocess-higgs.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.


## Training and Evaluating a Logistic Regression Model

Run the example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/example-higgs.py`

Command to run example using the pre-processed data in specific directory.
`python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/example-higgs.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

Command to run example on GPU 0:
python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/example-higgs.py --use_gpu

Command to run example on specified GPUs:
python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/example-higgs.py --use_gpu --device_ids 1 2

Command to run example with fit_intercept:
python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/example-higgs.py --use_gpu --fit_intercept

Command to collect training information:
```
python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/example-higgs.py --use_gpu --return_training_history=full

python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/example-higgs.py --use_gpu --return_training_history=summary
```
Now we should see the following output on P9+V100 System (--use_gpu --device_ids 0):

```
Data load time (s):  0.52
[pai4sk] Training time (s):  3.99
[pai4sk] Logarithmic loss:   0.6374
[pai4sk.lmodel] Training time (s):  3.97
[pai4sk.lmodel] Logarithmic loss:   0.6374
[sklearn] Training time (s):  65.38
[sklearn] Logarithmic loss:   0.6374
```

Times will vary depending on the system architecture. 

Note: By default the training will run on CPU.
With --use_gpu option, default device_ids list contains only GPU 0 for pai4sk and all the available GPUs for pai4sk.lmodel.


## Training and Evaluating a Decision Tree Model

Run the example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/example-higgs-decision-tree.py`

Command to run example using the pre-processed data in specific directory with 40 threads:
```
python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/example-higgs-decision-tree.py --data_path path/to/data --num_threads 40
```

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

The following results were obtained on a 40-core x86 machine (Intel(R) Xeon(R) Gold 6230 CPU @ 2.10GHz) with four V100 GPUs attached:
```
Data load time (s):  0.83
[snap.cpu] Training time (s):  11.28
[snap.cpu] Accuracy score:   0.7107
[snap.gpu] Training time (s):  6.67
[snap.gpu] Accuracy score:   0.7107
[sklearn] Training time (s):  677.50
[sklearn] Accuracy score:   0.7087
```

Times will vary depending on the system architecture. 

## Training and Evaluating a Random Forest Model

Run the example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/example-higgs-random-forest.py`

Command to run example using the pre-processed data in specific directory with 40 threads and 4 GPUs:
```
python $CONDA_PREFIX/pai4sk/local-examples/example-higgs/example-higgs-random-forest.py --data_path path/to/data --num_threads 40 --num_gpus 4
```

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

The following results were obtained on a 40-core x86 machine (Intel(R) Xeon(R) Gold 6230 CPU @ 2.10GHz) with four V100 GPUs attached:
```
Data load time (s):  0.93
[snap.cpu] Training time (s):  72.66
[snap.cpu] Accuracy score:   0.7034
[snap.gpu] Training time (s):  31.53
[snap.gpu] Accuracy score:   0.7030
[sklearn] Training time (s):  325.29
[sklearn] Accuracy score:   0.7025
```

Times will vary depending on the system architecture. 

&copy; Copyright IBM Corporation 2018, 2020
