# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from scipy import sparse

# Load the data
import time
import numpy as np
import argparse

from sklearn.metrics import mean_squared_error

defaultPath = "."
CLI=argparse.ArgumentParser()

CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--data_path",
    type=str,
    default=defaultPath
)

args = CLI.parse_args()

t0 = time.time()
X_train = np.load(args.data_path + "/data/housing.X_train.npy")
X_test  = np.load(args.data_path + "/data/housing.X_test.npy")
y_train = np.load(args.data_path + "/data/housing.y_train.npy")
y_test  = np.load(args.data_path + "/data/housing.y_test.npy")
print("Data load time (s):  {0:.2f}".format(time.time()-t0))

np.random.seed(182)
random_state = 42

tmp = X_train
tmp_y = y_train

# inflate the dataset 2000 times
for i in range(1999):
    tmp = np.vstack((tmp, X_train))
    tmp_y = np.hstack((tmp_y, y_train))

X_train = tmp
y_train = tmp_y

max_depth = 16


# Enable for regression with sample weighting
# sample_weight = np.random.randint(low=1, high=10, size=X_train.shape[0])
sample_weight = None


# Import snap DecisionreeRegressor from pai4sk module directly
from pai4sk import DecisionTreeRegressor
dt = DecisionTreeRegressor(random_state=random_state, max_depth=max_depth)
    
# Training time
t0 = time.time()
dt.fit(X_train, y_train, sample_weight=sample_weight)
print("[snap] Training time (s):  {0:.2f}".format(time.time()-t0))
       
# Inference
pred_test = dt.predict(X_test)
mse_snap = mean_squared_error(y_test, pred_test)
print("[snap] Mean squared error score:   {0:.4f}".format(mse_snap))



# pai4sk snap decision tree with histograms
dt = DecisionTreeRegressor(random_state=random_state, max_depth=max_depth, use_histograms=True, hist_nbins=64)

# Training time
t0 = time.time()
dt.fit(X_train, y_train, sample_weight=sample_weight)
print("[snap.histogram] Training time (s):  {0:.2f}".format(time.time()-t0))

# Inference
pred_test = dt.predict(X_test)
mse_snap = mean_squared_error(y_test, pred_test)
print("[snap.histogram] Mean squared error score:   {0:.4f}".format(mse_snap))



if not args.skip_sklearn:    
    # Import sklearn's DecisionTreeClassifier from pai4sk module directly
    from pai4sk.tree import DecisionTreeRegressor
    dt = DecisionTreeRegressor(random_state=random_state, max_depth=max_depth)
     
    # Training time
    t0 = time.time()
    dt.fit(X_train, y_train, sample_weight=sample_weight)
    print("[sklearn] Training time (s):  {0:.2f}".format(time.time()-t0))

    # Inference
    pred_test = dt.predict(X_test)

    # Evaluate accuracy_score on test set
    mse_sklearn = mean_squared_error(y_test, pred_test)
    print("[sklearn] Mean squared error score:   {0:.4f}".format(mse_sklearn))

