# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019, 2020. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function

import time
import argparse

# pai4sk explicit imports for snap_ml APIs
from pai4sk import MPI
from pai4sk import sml_io
from pai4sk import LogisticRegression as snapmlLR
from pai4sk.sml_metrics import log_loss as snapLOGLOSS, accuracy_score as snapACC

# pai4sk implicit API imports which use snap_ml or sklearn
from pai4sk.datasets import load_svmlight_file
from pai4sk.linear_model import LogisticRegression
from pai4sk.metrics import log_loss, accuracy_score

# pai4sk explicit imports for sklearn APIs
from pai4sk.linear_model import LogisticRegressionSklearn as sklearnLR
from pai4sk.metrics import accuracy_scoreSklearn,log_lossSklearn
from pai4sk.datasets import load_svmlight_fileSklearn

(comm_size, rank) = MPI.Comm_get_info()

defaultPath = "."

CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)
CLI.add_argument(
   "--device_ids",
   nargs="*",
   type=int,
   default=[]
)
CLI.add_argument(
    "--use_gpu",
    action = 'store_true',
    default=False
)

CLI.add_argument(
   "--use_balanced_class_weights",
   action = 'store_true',
   default=False
)
args = CLI.parse_args()

if args.use_balanced_class_weights:
  class_weight = "balanced"
else:
  class_weight = None

X_data, y_data = sml_io.load_svmlight_file(args.data_path + "/data/criteo.kaggle2014.train.svm")
snapml_lr = snapmlLR(device_ids=args.device_ids, use_gpu=args.use_gpu, 
                                   max_iter=50,class_weight=class_weight)

# Training
t0 = time.time()
snapml_lr.fit(X_data,y_data)
if (rank == 0):
    print("[pai4sk explicit] MPI process %d : Training time (s):  %.2f" % (rank, time.time()-t0))

proba = snapml_lr.predict_proba(X_data)
pred  = snapml_lr.predict(X_data)

acc1 = snapACC(y_data, pred)
log_loss1 = snapLOGLOSS(y_data, proba)
if (rank == 0):
    print("[pai4sk explicit] Accuracy:       %.4f" % (acc1))
    print("[pai4sk explicit] Log loss:       %.4f" % (log_loss1))


X_data, y_data = load_svmlight_file(args.data_path + "/data/criteo.kaggle2014.train.svm")
lr = LogisticRegression(dual=True, device_ids=args.device_ids, use_gpu=args.use_gpu,
                                   max_iter=50,class_weight=class_weight)

# Training
t0 = time.time()
lr.fit(X_data,y_data)
if (rank == 0):
    print("[pai4sk implicit] MPI process %d : Training time (s):  %.2f" % (rank, time.time()-t0))

proba = lr.predict_proba(X_data)
pred  = lr.predict(X_data)

acc2 = accuracy_score(y_data, pred)
log_loss2 = log_loss(y_data, proba)
if (rank == 0):
    print("[pai4sk implicit] Accuracy:       %.4f" % (acc2))
    print("[pai4sk implicit] Log loss:       %.4f" % (log_loss2))


X, y = load_svmlight_fileSklearn(args.data_path + "/data/criteo.kaggle2014.train.svm")

# Create Logistic Regression model
lr = sklearnLR(dual=True, fit_intercept=False, solver="liblinear")

t0 = time.time()
lr.fit(X,y)

if (rank == 0):
    print("[sklearn] Training time (s):  %.2f" % (time.time()-t0))

proba = lr.predict_proba(X)
pred  = lr.predict(X)


if (rank == 0):
    print("[sklearn] Accuracy:    %.4f" % (accuracy_scoreSklearn(y, pred)))
    print("[sklearn] Log loss:    %.4f" % (log_lossSklearn(y, proba)))

