# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function
import os

# Load the data
import argparse

from sklearn.datasets import load_svmlight_file
from sklearn.model_selection import train_test_split
import numpy as np
from sklearn.preprocessing import normalize

from snap_ml_mpi.Utils import dump_to_snap_format

defaultPath = os.getcwd()
CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)

args = CLI.parse_args()

print("Loading mnist8m dataset...")
X,y = load_svmlight_file(args.data_path + "/data/mnist8m.scale")

print('Splitting data...')
# Make the train-test split
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.25, random_state=42)

# Convert to numpy ararys
print('Converting to dense format...')
X_train = np.array(X_train.todense())
X_test  = np.array(X_test.todense())

# Normalize the training data
print('Normalizing data...')
X_train = normalize(X_train)
X_test  = normalize(X_test)

print('Saving data in sklearn format...')
# Save data to libsvm format for sklearn data loading
np.save(args.data_path + "/data/mnist8m.scale.norm.X_train.npy", X_train)
np.save(args.data_path + "/data/mnist8m.scale.norm.X_test.npy", X_test)
np.save(args.data_path + "/data/mnist8m.scale.norm.y_train.npy", y_train)
np.save(args.data_path + "/data/mnist8m.scale.norm.y_test.npy", y_test)

print('Saving data in distributed snap.ml format...')
# Save data in distributed snap.ml format
dump_to_snap_format(X_train, y_train, args.data_path + '/data/mnist8m.scale.norm.train.snap')
dump_to_snap_format(X_test, y_test, args.data_path + '/data/mnist8m.scale.norm.test.snap')
