# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function
import numpy as np
from scipy import sparse
from sklearn.datasets import dump_svmlight_file
from scipy import stats
import os
import argparse
from sklearn.preprocessing import normalize

from snap_ml_mpi.Utils import dump_to_snap_format


defaultPath = os.getcwd()
CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)
args = CLI.parse_args()


num_ex = 128
num_ft = 10240
density = 0.2

X = sparse.random(num_ex, num_ft, density,format='csr', data_rvs=stats.uniform(1,0).rvs)
y = 2*np.random.randint(2, size=num_ex)-1

X_norm = normalize(X, norm='l2', axis=1)

filename = args.data_path + "/data/small"

# Save data to libsvm format
with open(filename+".libsvm",'wb') as f:
    dump_svmlight_file(X_norm,y,f,zero_based=False)

# Save data to csv format
X_dense = np.array(X.todense())
csv_data = np.column_stack((y,X_dense))
np.savetxt(filename + '.csv', csv_data, delimiter=',')

# Save data to snap dense format
dump_to_snap_format(X_dense, y, filename+'_dense.snap')
dump_to_snap_format(X_dense, y, filename+'_dense.snap.t', transpose=True)
# Save data to snap sparse format
dump_to_snap_format(X, y, filename+'_sparse.snap')
dump_to_snap_format(X, y, filename+'_sparse.snap.t', transpose=True)

